
YarkoCooldowns = {}

YarkoCooldowns.DefaultMainColor = NORMAL_FONT_COLOR;
YarkoCooldowns.DefaultFlash = "Y";
YarkoCooldowns.DefaultFlashSeconds = 3;
YarkoCooldowns.DefaultAlternate = 1;
YarkoCooldowns.DefaultFlashColor = { r = 1.0, g = 0, b = 0 };
YarkoCooldowns.DefaultFontLocation = "Fonts";
YarkoCooldowns.DefaultFontFile = "FRIZQT__.TTF";
YarkoCooldowns.DefaultFontHeight = 18;
YarkoCooldowns.DefaultShadow = "N";
YarkoCooldowns.DefaultOutline = 3;
YarkoCooldowns.DefaultTenths = "N";
YarkoCooldowns.DefaultBelowTwo = "N";
YarkoCooldowns.DefaultSeconds = 60;

YarkoCooldowns.Cooldowns = {};


local Counters = {};

local Scales = {
	[1] = 1.4,
	[2] = 1.2,
	[3] = .9,
	[4] = .8
};

local TimeSinceLastUpdate = 1;
local UpdateInterval = .05;

local ButtonWidth = 28;
local OutlineList = {nil, "OUTLINE", "THICKOUTLINE"};


function YarkoCooldowns.OnLoad(self)
	self:RegisterEvent("PLAYER_LOGIN");
end


function YarkoCooldowns.OnEvent(self, event, ...)
	if (event == "PLAYER_LOGIN") then
        YarkoCooldowns.OptionsSetup();
        
		hooksecurefunc("CooldownFrame_SetTimer", YarkoCooldowns.CooldownSetTimer);
		hooksecurefunc("ActionButton_OnUpdate", YarkoCooldowns.ActionButtonUpdate);
	end
end


function YarkoCooldowns.CooldownSetTimer(self, start, duration, enable)
	if (start and self.mark ~= 0) then
		local name = self:GetName();
		
		if (strsub(name, 1, 12) ~= "ActionButton" and strsub(name, 1, 8) ~= "MultiBar") then
			YarkoCooldowns.StartCooldown(self, start, duration, enable);
		end
	end
end


function YarkoCooldowns.ActionButtonUpdate(self, elapsed)
	if (self.cooldown.mark ~= 0) then
		local start, duration, enable, charges = GetActionCooldown(self.action);
		
		if (start) then
			if (start + duration ~= self.finish or enable ~= self.enable or charges ~= self.charges) then
				self.finish = start + duration;
				self.enable = enable;
				self.charges = charges;
				
				if (charges == 0) then
					YarkoCooldowns.StartCooldown(self.cooldown, start, duration, enable);
				end
			end
		end
	end
end


function YarkoCooldowns.StartCooldown(self, start, duration, enable)
	local name = self:GetName();
	
	if (not self.mark) then
		if (not name) then
			self.mark = 0;
			return;
		end
		
		self.mark = (((self:GetParent():GetWidth() >= ButtonWidth 
			or self:GetParent():GetParent():GetName() == "WatchFrameLines"
			or self:GetParent():GetParent():GetName() == "SpellFlyout")
			and 1) or 0);
		--self.mark = 1;
	end
	
	if (self.mark == 1) then
		local countername = "YarkoCooldowns_"..name.."Counter";
		
		if (start > 0 and duration > 1.5 and enable > 0) then
			-- For Bartender, don't parent counter to cooldown so that button can be made trasparent and cooldown still visible
			local specialaddon = Bartender4;
				
			if (not _G[countername]) then
				local frame = CreateFrame("Frame", countername, 
					((not specialaddon) and self) or nil, "YarkoCooldowns_CounterTemplate");
				frame:SetPoint("CENTER", self:GetParent(), "CENTER", 1, -2);
				
				if (specialaddon) then
					frame:SetFrameStrata(self:GetParent():GetFrameStrata());
				end
				
				frame:SetFrameLevel(self:GetFrameLevel() + 5);
				frame:SetToplevel(true);
				
				self:HookScript("OnShow", function(self) _G["YarkoCooldowns_"..self:GetName().."Counter"]:Show(); end);
				self:HookScript("OnHide", function(self) _G["YarkoCooldowns_"..self:GetName().."Counter"]:Hide(); end);
				
				YarkoCooldowns.UpdateFont(name);
				frame:Show();
				
				tinsert(YarkoCooldowns.Cooldowns, name); 
			end
			
			Counters[name] = {};
			Counters[name].start = start;
			Counters[name].duration = duration;
			Counters[name].enable = enable;
			Counters[name].cooldown = self;
			Counters[name].counterframe = _G[countername];
			Counters[name].scale = self:GetParent():GetWidth() / ActionButton1:GetWidth() 
				* ((specialaddon and self:GetParent():GetEffectiveScale()) or 1);
			Counters[name].flag = false;
			Counters[name].accum = 0;
		else
			if (_G[countername]) then
				_G[countername.."Text"]:SetText("");
			end
			
			if (Counters[name]) then
				Counters[name] = nil;
			end
		end
	end
end


function YarkoCooldowns.OnUpdate(self, elapsed)
	TimeSinceLastUpdate = TimeSinceLastUpdate + elapsed;
	
	if (TimeSinceLastUpdate > UpdateInterval) then
		for k, v in pairs(Counters) do
			local timeleft = v.start + v.duration - GetTime();
			local counterframe = v.counterframe;
			local countertext = _G["YarkoCooldowns_"..k.."CounterText"];
			
			if (timeleft > 0 and v.enable > 0) then
				if (_G[k]:IsVisible()) then
					local timeleftstr, str, length;
					
					if (timeleft > 3600) then
						timeleftstr = ((YarkoCooldowns_SavedVars.Tenths == "Y" 
							and YarkoCooldowns.TrimZeros(format("%.1f", (timeleft + 180) / 3600)))
							or format("%d", ceil(timeleft / 3600)));
						str = "h";
					elseif (timeleft > YarkoCooldowns_SavedVars.Seconds) then
						timeleftstr = ((YarkoCooldowns_SavedVars.Tenths == "Y" 
							and YarkoCooldowns.TrimZeros(format("%.1f", (timeleft + 3) / 60)))
							or format("%d", ceil(timeleft / 60)));
						str = "m";
					else
						timeleftstr = ((timeleft <= 2 and YarkoCooldowns_SavedVars.BelowTwo == "Y" 
							and format("%.1f", timeleft + 0.05))
							or format("%d", ceil(timeleft)));
					end
					
					length = strlen(timeleftstr);
					
					if (str) then
						if (length > 2) then
							timeleftstr = timeleftstr.."\n"..str;
						else
							timeleftstr = timeleftstr..str;
							length = length + 1;
						end
					end
					
					if (timeleftstr ~= countertext:GetText()) then
						if (length > 4) then
							length = 4;
						end
						
						counterframe:SetScale(Scales[length] * v.scale);
						countertext:SetText(timeleftstr);
					end
					
					if (timeleft <= YarkoCooldowns_SavedVars.FlashSeconds and YarkoCooldowns_SavedVars.Flash == "Y") then
						if (YarkoCooldowns_SavedVars.Alternate == 1) then
							v.accum = v.accum - 1;
							
							if (v.accum < 1) then
								v.flag = not v.flag;
								v.accum = 4;
							end
						else
							if (not v.flag) then
								v.flag = true;
								v.oldflag = false;
							end
						end
					else
						if (v.flag) then
							v.flag = false;
							v.oldflag = true;
						end
						
						if (v.accum > 0) then
							v.accum = 0;
						end
					end
					
					if (v.flag ~= v.oldflag) then
						YarkoCooldowns.SetCounterColor(countertext, v.flag);
						v.oldflag = v.flag;
					end
				else
					countertext:SetText("");
				end
			else
				Counters[k] = nil;
				countertext:SetText("");
			end
		end

		TimeSinceLastUpdate = 0;
	end
end


function YarkoCooldowns.TrimZeros(instr)
	local outstr = "";
	local str = "";
	local i;
	
	for i = 1, strlen(instr) do
		if (strsub(instr, i, i) ~= "0") then
			str = strsub(instr, i);
			break;
		end
	end
	
	for i = strlen(str), 1, -1 do
		if (strsub(str, i, i) == ".") then
			outstr = strsub(str, 1, i - 1);
			break;
		end
		
		if (strsub(str, i, i) ~= "0") then
			outstr = strsub(str, 1, i);
			break;
		end
	end
	
	return outstr;
end


function YarkoCooldowns.UpdateFont(cooldownname)
	local countertext = _G["YarkoCooldowns_"..cooldownname.."CounterText"];
	
	if (YarkoCooldowns_SavedVars.Shadow == "Y") then
		countertext:SetShadowOffset(1, -1);
	else
		countertext:SetShadowOffset(0, 0);
	end
	
	countertext:SetFont(YarkoCooldowns_SavedVars.FontLocation.."\\"..YarkoCooldowns_SavedVars.FontFile, 
		YarkoCooldowns_SavedVars.FontHeight, OutlineList[YarkoCooldowns_SavedVars.Outline]);
		
	local counter = Counters[cooldownname];
	YarkoCooldowns.SetCounterColor(countertext, (counter and counter.flag) or false)
end


function YarkoCooldowns.SetCounterColor(countertext, flag)
	if (not flag) then
		countertext:SetTextColor(YarkoCooldowns_SavedVars.MainColor.r, 
			YarkoCooldowns_SavedVars.MainColor.g, YarkoCooldowns_SavedVars.MainColor.b);
	else
		countertext:SetTextColor(YarkoCooldowns_SavedVars.FlashColor.r, 
			YarkoCooldowns_SavedVars.FlashColor.g, YarkoCooldowns_SavedVars.FlashColor.b);
	end
end
