
---------------------------------------------------
-- Global vars
---------------------------------------------------

YarkoCooldowns_SavedVars = {};


---------------------------------------------------
-- Local vars
---------------------------------------------------

local InGameFonts = {
	"2002.ttf",
	"2002B.ttf",
	"ARHei.ttf",
	"ARIALN.TTF",
	"ARKai_C.ttf",
	"ARKai_T.ttf",
	"bHEI00M.ttf",
	"bHEI01B.ttf",
	"bKAI00M.ttf",
	"bLEI00D.ttf",
	"FRIZQT__.TTF",
	"FRIZQT___CYR.TTF",
	"K_Damage.TTF",
	"K_Pagetext.TTF",
	"MORPHEUS.TTF",
	"MORPHEUS_CYR.TTF",
	"NIM_____.ttf",
	"SKURRI.TTF",
	"SKURRI_CYR.TTF"
};

local IsDropdownCounting = false;
local DropdownTimer = 0;
local MenuResized = false;


---------------------------------------------------
-- YarkoCooldowns.OptionsSetup
---------------------------------------------------
function YarkoCooldowns.OptionsSetup()
	if (not YarkoCooldowns_SavedVars.Tenths) then
		YarkoCooldowns.OptionsDefault();
	end
	if (not YarkoCooldowns_SavedVars.Flash) then
		YarkoCooldowns_SavedVars.Flash = YarkoCooldowns.DefaultFlash;
	end
	if (not YarkoCooldowns_SavedVars.FlashSeconds) then
		YarkoCooldowns_SavedVars.FlashSeconds = YarkoCooldowns.DefaultFlashSeconds;
	end
	if (not YarkoCooldowns_SavedVars.Alternate) then
		YarkoCooldowns_SavedVars.Alternate = YarkoCooldowns.DefaultAlternate;
	end
	
	InterfaceOptions_AddCategory(YarkoCooldowns_OptionsPanel);
	
	-- Set scroll frame so that it resets the dropdown hide counter
	YarkoCooldowns_FontDropDownScrollFrameScrollBar:SetScript("OnEnter", YarkoCooldowns.StopCounting);
	YarkoCooldowns_FontDropDownScrollFrameScrollBar:SetScript("OnLeave", YarkoCooldowns.StartCounting);
	YarkoCooldowns_FontDropDownScrollFrameScrollBarScrollUpButton:SetScript("OnEnter", YarkoCooldowns.StopCounting);
	YarkoCooldowns_FontDropDownScrollFrameScrollBarScrollUpButton:SetScript("OnLeave", YarkoCooldowns.StartCounting);
	YarkoCooldowns_FontDropDownScrollFrameScrollBarScrollDownButton:SetScript("OnEnter", YarkoCooldowns.StopCounting);
	YarkoCooldowns_FontDropDownScrollFrameScrollBarScrollDownButton:SetScript("OnLeave", YarkoCooldowns.StartCounting);
	
	-- Create invis buttons for scrollbar to hide dropdown
	CreateFrame("Button", "YarkoCooldowns_UpButton", YarkoCooldowns_FontDropDownScrollFrameScrollBarScrollUpButton,
		"YarkoCooldowns_InvisibleButtonTemplate");
	YarkoCooldowns_UpButton:SetAllPoints(YarkoCooldowns_FontDropDownScrollFrameScrollBarScrollUpButton);
	CreateFrame("Button", "YarkoCooldowns_DownButton", YarkoCooldowns_FontDropDownScrollFrameScrollBarScrollDownButton,
		"YarkoCooldowns_InvisibleButtonTemplate");
	YarkoCooldowns_DownButton:SetAllPoints(YarkoCooldowns_FontDropDownScrollFrameScrollBarScrollDownButton);
		
	-- Adjust dropdown field widths
	UIDropDownMenu_SetWidth(YarkoCooldowns_OptionsPanelAlternate, 67)
	UIDropDownMenu_SetWidth(YarkoCooldowns_OptionsPanelOutline, 60)
end


---------------------------------------------------
-- YarkoCooldowns.OptionsDefault
---------------------------------------------------
function YarkoCooldowns.OptionsDefault()
	YarkoCooldowns_SavedVars.MainColor = YarkoCooldowns.CopyColors(YarkoCooldowns.DefaultMainColor);
	YarkoCooldowns_SavedVars.Flash = YarkoCooldowns.DefaultFlash;
	YarkoCooldowns_SavedVars.FlashSeconds = YarkoCooldowns.DefaultFlashSeconds;
	YarkoCooldowns_SavedVars.Alternate = YarkoCooldowns.DefaultAlternate;
	YarkoCooldowns_SavedVars.FlashColor = YarkoCooldowns.CopyColors(YarkoCooldowns.DefaultFlashColor);
	YarkoCooldowns_SavedVars.FontLocation = YarkoCooldowns.DefaultFontLocation;
	YarkoCooldowns_SavedVars.FontFile = YarkoCooldowns.DefaultFontFile;
	YarkoCooldowns_SavedVars.FontHeight = YarkoCooldowns.DefaultFontHeight;
	YarkoCooldowns_SavedVars.Shadow = YarkoCooldowns.DefaultShadow;
	YarkoCooldowns_SavedVars.Outline = YarkoCooldowns.DefaultOutline;
	YarkoCooldowns_SavedVars.Tenths = YarkoCooldowns.DefaultTenths;
	YarkoCooldowns_SavedVars.BelowTwo = YarkoCooldowns.DefaultBelowTwo;
	YarkoCooldowns_SavedVars.Seconds = YarkoCooldowns.DefaultSeconds;
	
	YarkoCooldowns.UpdateCooldowns();
end


---------------------------------------------------
-- YarkoCooldowns.OptionsRefresh
---------------------------------------------------
function YarkoCooldowns.OptionsRefresh()
	YarkoCooldowns_OptionsPanelMainColorNormalTexture:SetVertexColor(YarkoCooldowns_SavedVars.MainColor.r, 
		YarkoCooldowns_SavedVars.MainColor.g, YarkoCooldowns_SavedVars.MainColor.b);
	YarkoCooldowns_OptionsPanelFlash:SetChecked(YarkoCooldowns_SavedVars.Flash == "Y");
	YarkoCooldowns_OptionsPanelFlashSeconds:SetText(YarkoCooldowns_SavedVars.FlashSeconds);
	YarkoCooldowns_OptionsPanelFlashSeconds:SetCursorPosition(0);
    UIDropDownMenu_Initialize(YarkoCooldowns_OptionsPanelAlternate, YarkoCooldowns.AlternateDropDownInit);
	UIDropDownMenu_SetSelectedValue(YarkoCooldowns_OptionsPanelAlternate, YarkoCooldowns_SavedVars.Alternate);
	YarkoCooldowns_OptionsPanelFlashColorNormalTexture:SetVertexColor(YarkoCooldowns_SavedVars.FlashColor.r, 
		YarkoCooldowns_SavedVars.FlashColor.g, YarkoCooldowns_SavedVars.FlashColor.b);
	YarkoCooldowns_OptionsPanelFontLocation:SetText(YarkoCooldowns_SavedVars.FontLocation);
	YarkoCooldowns_OptionsPanelFontLocation:SetCursorPosition(0);
	YarkoCooldowns_OptionsPanelFontFile:SetText(YarkoCooldowns_SavedVars.FontFile);
	YarkoCooldowns_OptionsPanelFontFile:SetCursorPosition(0);
	YarkoCooldowns_OptionsPanelFontHeight:SetNumber(YarkoCooldowns_SavedVars.FontHeight);
	YarkoCooldowns_OptionsPanelFontHeight:SetCursorPosition(0);
	YarkoCooldowns_OptionsPanelShadow:SetChecked(YarkoCooldowns_SavedVars.Shadow == "Y");
    UIDropDownMenu_Initialize(YarkoCooldowns_OptionsPanelOutline, YarkoCooldowns.OutlineDropDownInit);
	UIDropDownMenu_SetSelectedValue(YarkoCooldowns_OptionsPanelOutline, YarkoCooldowns_SavedVars.Outline);
	YarkoCooldowns_OptionsPanelTenths:SetChecked(YarkoCooldowns_SavedVars.Tenths == "Y");
	YarkoCooldowns_OptionsPanelBelowTwo:SetChecked(YarkoCooldowns_SavedVars.BelowTwo == "Y");
	YarkoCooldowns_OptionsPanelSeconds:SetText(YarkoCooldowns_SavedVars.Seconds);
	YarkoCooldowns_OptionsPanelSeconds:SetCursorPosition(0);
end


---------------------------------------------------
-- YarkoCooldowns.OptionsOkay
---------------------------------------------------
function YarkoCooldowns.OptionsOkay()
	YarkoCooldowns_SavedVars.MainColor.r, YarkoCooldowns_SavedVars.MainColor.g, 
		YarkoCooldowns_SavedVars.MainColor.b 
		= YarkoCooldowns_OptionsPanelMainColorNormalTexture:GetVertexColor();
	YarkoCooldowns_SavedVars.Flash = ((YarkoCooldowns_OptionsPanelFlash:GetChecked() and "Y") or "N");
	YarkoCooldowns_SavedVars.FlashSeconds = YarkoCooldowns_OptionsPanelFlashSeconds:GetNumber();
	YarkoCooldowns_SavedVars.Alternate = UIDropDownMenu_GetSelectedValue(YarkoCooldowns_OptionsPanelAlternate);
	YarkoCooldowns_SavedVars.FlashColor.r, YarkoCooldowns_SavedVars.FlashColor.g, 
		YarkoCooldowns_SavedVars.FlashColor.b 
		= YarkoCooldowns_OptionsPanelFlashColorNormalTexture:GetVertexColor();
	YarkoCooldowns_SavedVars.FontLocation = YarkoCooldowns_OptionsPanelFontLocation:GetText();
	YarkoCooldowns_SavedVars.FontFile = YarkoCooldowns_OptionsPanelFontFile:GetText();
	YarkoCooldowns_SavedVars.FontHeight = YarkoCooldowns_OptionsPanelFontHeight:GetNumber();
	YarkoCooldowns_SavedVars.Shadow = ((YarkoCooldowns_OptionsPanelShadow:GetChecked() and "Y") or "N");
	YarkoCooldowns_SavedVars.Outline = UIDropDownMenu_GetSelectedValue(YarkoCooldowns_OptionsPanelOutline);
	YarkoCooldowns_SavedVars.Tenths = ((YarkoCooldowns_OptionsPanelTenths:GetChecked() and "Y") or "N");
	YarkoCooldowns_SavedVars.BelowTwo = ((YarkoCooldowns_OptionsPanelBelowTwo:GetChecked() and "Y") or "N");
	YarkoCooldowns_SavedVars.Seconds = YarkoCooldowns_OptionsPanelSeconds:GetNumber();
		
	if (YarkoCooldowns_SavedVars.FlashSeconds < 0) then
		YarkoCooldowns_SavedVars.FlashSeconds = 0;
	end

	if (YarkoCooldowns_SavedVars.FontHeight < 4) then
		YarkoCooldowns_SavedVars.FontHeight = 4;
	end
	
	if (YarkoCooldowns_SavedVars.FontHeight > 72) then
		YarkoCooldowns_SavedVars.FontHeight = 72;
	end
	
	--if (YarkoCooldowns_SavedVars.Seconds > 120) then
	--	YarkoCooldowns_SavedVars.Seconds = 120;
	--end

	if (YarkoCooldowns_SavedVars.Seconds < 10) then
		YarkoCooldowns_SavedVars.Seconds = 10;
	end
	
	YarkoCooldowns.UpdateCooldowns();
end


---------------------------------------------------
-- YarkoCooldowns.OutlineDropDownInit
---------------------------------------------------
function YarkoCooldowns.OutlineDropDownInit(self)
	local info = UIDropDownMenu_CreateInfo();
	
	info.text = YARKOCOOLDOWNS_CONFIG_NONE;
	info.value = 1;
	info.func = YarkoCooldowns.OutlineDropDownOnClick;
	info.checked = nil;
	UIDropDownMenu_AddButton(info);
	
	info.text = YARKOCOOLDOWNS_CONFIG_NORMAL;
	info.value = 2;
	info.func = YarkoCooldowns.OutlineDropDownOnClick;
	info.checked = nil;
	UIDropDownMenu_AddButton(info);
	
	info.text = YARKOCOOLDOWNS_CONFIG_THICK;
	info.value = 3;
	info.func = YarkoCooldowns.OutlineDropDownOnClick;
	info.checked = nil;
	UIDropDownMenu_AddButton(info);
end


---------------------------------------------------
-- YarkoCooldowns.OutlineDropDownOnClick
---------------------------------------------------
function YarkoCooldowns.OutlineDropDownOnClick(self)
	UIDropDownMenu_SetSelectedValue(YarkoCooldowns_OptionsPanelOutline, self.value);
end


---------------------------------------------------
-- YarkoCooldowns.AlternateDropDownInit
---------------------------------------------------
function YarkoCooldowns.AlternateDropDownInit(self)
	local info = UIDropDownMenu_CreateInfo();
	
	info.text = YARKOCOOLDOWNS_CONFIG_ALTFLASH;
	info.value = 1;
	info.func = YarkoCooldowns.AlternateDropDownOnClick;
	info.checked = nil;
	UIDropDownMenu_AddButton(info);
	
	info.text = YARKOCOOLDOWNS_CONFIG_ALTSOLID;
	info.value = 2;
	info.func = YarkoCooldowns.AlternateDropDownOnClick;
	info.checked = nil;
	UIDropDownMenu_AddButton(info);
end


---------------------------------------------------
-- YarkoCooldowns.AlternateDropDownOnClick
---------------------------------------------------
function YarkoCooldowns.AlternateDropDownOnClick(self)
	UIDropDownMenu_SetSelectedValue(YarkoCooldowns_OptionsPanelAlternate, self.value);
end


---------------------------------------------------
-- YarkoCooldowns.ToggleFontDropDown()
---------------------------------------------------
function YarkoCooldowns.ToggleFontDropDown()
	if (YarkoCooldowns_FontDropDown:IsShown()) then
		YarkoCooldowns_FontDropDown:Hide();
	else
		YarkoCooldowns_FontDropDown:Show();
		FauxScrollFrame_SetOffset(YarkoCooldowns_FontDropDownScrollFrame, 0);
		YarkoCooldowns_FontDropDownScrollFrame:SetVerticalScroll(0);
	end
end


---------------------------------------------------
-- YarkoCooldowns.FontDropDownOnShow
---------------------------------------------------
function YarkoCooldowns.FontDropDownOnShow(self)
	YarkoCooldowns.FontDropDownUpdate();
	--YarkoCooldowns.StartCounting();
end


---------------------------------------------------
-- YarkoCooldowns.FontDropDownUpdate
---------------------------------------------------
function YarkoCooldowns.FontDropDownUpdate()
	local t, index;
	local numButtons = #InGameFonts;
	local button, buttonText;
	local maxwidth = 0;
	
	if (numButtons > 15) then
		numButtons = 15;
	end
	
	local offset = FauxScrollFrame_GetOffset(YarkoCooldowns_FontDropDownScrollFrame);
	
	for t = 1, 15 do
		index = t + offset;
		button = _G["YarkoCooldowns_FontDropDownButton"..t];
		
		if (index <= #InGameFonts) then
			buttonText = _G["YarkoCooldowns_FontDropDownButton"..t.."NormalText"];
			button:SetText(InGameFonts[index]);
			width = buttonText:GetWidth() + 10;
			
			if (width > maxwidth) then
				maxwidth = width;
			end
			
			button:Show();
		else
			button:Hide();
		end
	end
	
	-- Adjust menu width
	if (not MenuResized) then
		for t = 1,  15 do
			button = _G["YarkoCooldowns_FontDropDownButton"..t];
			button:SetWidth(maxwidth);
		end
		
		if (#InGameFonts > 15) then
			maxwidth = maxwidth + 15;
		end
		
		YarkoCooldowns_FontDropDown:SetWidth(maxwidth + 25);
		
		MenuResized = true;
	
		-- Adjust menu height
		YarkoCooldowns_FontDropDown:SetHeight((numButtons * 16) + 27);
	end
	
	if (#InGameFonts > 15) then
		-- Show or hide scrollbar invis buttons
		if (offset == 0) then
			YarkoCooldowns_UpButton:Show();
		else
			YarkoCooldowns_UpButton:Hide();
		end

		if (offset == #InGameFonts - 15) then
			YarkoCooldowns_DownButton:Show();
		else
			YarkoCooldowns_DownButton:Hide();
		end
	end

	-- ScrollFrame stuff
	FauxScrollFrame_Update(YarkoCooldowns_FontDropDownScrollFrame, #InGameFonts, 15, 16);
end


---------------------------------------------------
-- YarkoCooldowns.FontDropDownOnUpdate
---------------------------------------------------
function YarkoCooldowns.FontDropDownButtonOnClick(self, button)
	YarkoCooldowns_OptionsPanelFontFile:SetText(_G[self:GetName().."NormalText"]:GetText());
	YarkoCooldowns_FontDropDown:Hide();
end


---------------------------------------------------
-- YarkoCooldowns.FontDropDownOnUpdate
-- Handle dropdown counting
---------------------------------------------------
function YarkoCooldowns.FontDropDownOnUpdate(self, elapsed)
	-- Hide dropdown after 2 seconds
	if (IsDropdownCounting) then
		DropdownTimer = DropdownTimer - elapsed;
		
		if (DropdownTimer < 0) then
			YarkoCooldowns_FontDropDown:Hide();
			IsDropdownCounting = false;
		end
	end
end

---------------------------------------------------
-- YarkoCooldowns.StartCounting
-- Start the dropdown hide countdown 
---------------------------------------------------
function YarkoCooldowns.StartCounting()
	DropdownTimer = UIDROPDOWNMENU_SHOW_TIME;
	IsDropdownCounting = true;
end


---------------------------------------------------
-- YarkoCooldowns.StopCounting
-- Stop the dropdown hide countdown 
---------------------------------------------------
function YarkoCooldowns.StopCounting()
	IsDropdownCounting = false;
end


---------------------------------------------------
-- YarkoCooldowns.SwatchOnClick
---------------------------------------------------
function YarkoCooldowns.SwatchOnClick(self)
	local info = {};
	info.extraInfo = _G[self:GetName().."NormalTexture"];
	info.r, info.g, info.b = info.extraInfo:GetVertexColor();
	info.swatchFunc = YarkoCooldowns.SetColor;
	OpenColorPicker(info);
end


---------------------------------------------------
-- YarkoCooldowns.SetColor
---------------------------------------------------
function YarkoCooldowns.SetColor()
	if (not ColorPickerFrame:IsVisible()) then
		ColorPickerFrame.extraInfo:SetVertexColor(ColorPickerFrame:GetColorRGB());
	end
end


---------------------------------------------------
-- YarkoCooldowns.UpdateCooldowns
---------------------------------------------------
function YarkoCooldowns.UpdateCooldowns()
	local index, value;
	
	for index, value in ipairs(YarkoCooldowns.Cooldowns) do
		YarkoCooldowns.UpdateFont(value);
	end
end


---------------------------------------------------
-- YarkoCooldowns.CopyColors
---------------------------------------------------
function YarkoCooldowns.CopyColors(object)
	local tbl = {};
	
	tbl.r = object.r;
	tbl.g = object.g;
	tbl.b = object.b;
	
	return tbl;
end
